@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007 Derick Rethans      |
+   | Copyright (c) 2002-2010 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -20,10 +20,12 @@
 #define PHP_XDEBUG_H
 
 #define XDEBUG_NAME       "Xdebug"
-#define XDEBUG_VERSION    "2.0.5"
+#define XDEBUG_VERSION    "2.1.0"
 #define XDEBUG_AUTHOR     "Derick Rethans"
-#define XDEBUG_COPYRIGHT  "Copyright (c) 2002-2009 by Derick Rethans"
+#define XDEBUG_COPYRIGHT  "Copyright (c) 2002-2010 by Derick Rethans"
+#define XDEBUG_COPYRIGHT_SHORT "Copyright (c) 2002-2010"
 #define XDEBUG_URL        "http://xdebug.org"
+#define XDEBUG_URL_FAQ    "http://xdebug.org/docs/faq#api"
 
 #include "php.h"
 
@@ -46,6 +48,8 @@ extern zend_module_entry xdebug_module_entry;
 #include "TSRM.h"
 #endif
 
+#include "main/SAPI.h"
+
 #if MEMORY_LIMIT
 # define HAVE_PHP_MEMORY_USAGE 1
 #elif PHP_VERSION_ID >= 50201
@@ -62,18 +66,25 @@ extern zend_module_entry xdebug_module_entry;
 # define XG_MEMORY_PEAK_USAGE()	AG(allocated_memory_peak)
 #endif
 
+#if PHP_VERSION_ID >= 50300
+# define XG_SAPI_HEADER_OP_DC   , sapi_header_op_enum op
+# define XG_SAPI_HEADER_OP_CC   , op
+#else
+# define XG_SAPI_HEADER_OP_DC
+# define XG_SAPI_HEADER_OP_CC
+#endif
+
 PHP_MINIT_FUNCTION(xdebug);
 PHP_MSHUTDOWN_FUNCTION(xdebug);
 PHP_RINIT_FUNCTION(xdebug);
 PHP_RSHUTDOWN_FUNCTION(xdebug);
 PHP_MINFO_FUNCTION(xdebug);
-#ifdef ZEND_ENGINE_2
 ZEND_MODULE_POST_ZEND_DEACTIVATE_D(xdebug);
-#endif
 
 /* call stack functions */
 PHP_FUNCTION(xdebug_get_stack_depth);
 PHP_FUNCTION(xdebug_get_function_stack);
+PHP_FUNCTION(xdebug_get_formatted_function_stack);
 PHP_FUNCTION(xdebug_print_function_stack);
 PHP_FUNCTION(xdebug_get_declared_vars);
 PHP_FUNCTION(xdebug_call_class);
@@ -100,6 +111,11 @@ PHP_FUNCTION(xdebug_start_trace);
 PHP_FUNCTION(xdebug_stop_trace);
 PHP_FUNCTION(xdebug_get_tracefile_name);
 
+/* error collecting functions */
+PHP_FUNCTION(xdebug_start_error_collection);
+PHP_FUNCTION(xdebug_stop_error_collection);
+PHP_FUNCTION(xdebug_get_collected_errors);
+
 /* profiling functions */
 PHP_FUNCTION(xdebug_get_profiler_filename);
 PHP_FUNCTION(xdebug_dump_aggr_profiling_data);
@@ -107,6 +123,7 @@ PHP_FUNCTION(xdebug_clear_aggr_profiling_data);
 
 /* misc functions */
 PHP_FUNCTION(xdebug_dump_superglobals);
+PHP_FUNCTION(xdebug_get_headers);
 #if HAVE_PHP_MEMORY_USAGE
 PHP_FUNCTION(xdebug_memory_usage);
 PHP_FUNCTION(xdebug_peak_memory_usage);
@@ -125,6 +142,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	long          collect_params;
 	zend_bool     collect_return;
 	zend_bool     collect_vars;
+	zend_bool     collect_assignments;
 	zend_bool     extended_info;
 	zend_bool     show_ex_trace;
 	zend_bool     show_local_vars;
@@ -133,8 +151,13 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	double        start_time;
 	HashTable    *active_symbol_table;
 	zend_execute_data *active_execute_data;
+	zend_op_array     *active_op_array;
+	zval              *This;
 	unsigned int  prev_memory;
+	char         *file_link_format;
 
+	zend_bool     overload_var_dump;
+	zend_bool     var_dump_overloaded;
 	void        (*orig_var_dump_func)(INTERNAL_FUNCTION_PARAMETERS);
 	void        (*orig_set_time_limit_func)(INTERNAL_FUNCTION_PARAMETERS);
 
@@ -147,6 +170,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	long          trace_format;
 	char         *tracefile_name;
 	char         *last_exception_trace;
+	char         *last_eval_statement;
 
 	/* variable dumping limitation settings */
 	long          display_max_children;
@@ -161,6 +185,10 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	unsigned int  function_count;
 	int           reserved_offset;
 
+	/* used for collection errors */
+	zend_bool     do_collect_errors;
+	xdebug_llist *collected_errors;
+
 	/* superglobals */
 	zend_bool     dump_globals;
 	zend_bool     dump_once;
@@ -175,6 +203,9 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	xdebug_llist  request;
 	xdebug_llist  session;
 
+	/* headers */
+	xdebug_llist *headers;
+
 	/* remote settings */
 	zend_bool     remote_enable;  /* 0 */
 	long          remote_port;    /* 9000 */
@@ -182,8 +213,10 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	long          remote_mode;    /* XDEBUG_NONE, XDEBUG_JIT, XDEBUG_REQ */
 	char         *remote_handler; /* php3, gdb, dbgp */
 	zend_bool     remote_autostart; /* Disables the requirement for XDEBUG_SESSION_START */
+	zend_bool     remote_connect_back;   /* connect back to the HTTP requestor */
 	char         *remote_log;       /* Filename to log protocol communication to */
 	FILE         *remote_log_file;  /* File handler for protocol log */
+	long          remote_cookie_expire_time; /* Expire time for the remote-session cookie */
 
 	char         *ide_key;    /* from environment, USER, USERNAME or empty */
 
@@ -220,6 +253,8 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	HashTable  aggr_calls;
 	zend_bool  profiler_aggregate;
 
+	/* scream */
+	zend_bool  do_scream;
 ZEND_END_MODULE_GLOBALS(xdebug)
 
 #ifdef ZTS
