@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007 Derick Rethans      |
+   | Copyright (c) 2002-2010 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -533,9 +533,9 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 				
 					/* we do not always have script_name available, so if we
 					 * don't have it and this format specifier is used then we
-					 * make this function fail by returning 0. */
+					 * simple do nothing for this specifier */
 					if (!script_name) {
-						return 0;
+						break;
 					}
 
 					/* create a copy to work on */
