@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007 Derick Rethans      |
+   | Copyright (c) 2002-2010 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,6 +22,7 @@
 #include "xdebug_var.h"
 #include "xdebug_code_coverage.h"
 #include "xdebug_compat.h"
+#include "xdebug_tracing.h"
 
 extern ZEND_DECLARE_MODULE_GLOBALS(xdebug);
 
@@ -41,6 +42,228 @@ void xdebug_coverage_file_dtor(void *data)
 	xdfree(file);
 }
 
+#define XDEBUG_OPCODE_OVERRIDE(f) \
+	int xdebug_##f##_handler(ZEND_OPCODE_HANDLER_ARGS) \
+	{ \
+		return xdebug_common_override_handler(ZEND_OPCODE_HANDLER_ARGS_PASSTHRU); \
+	}
+
+
+int xdebug_common_override_handler(ZEND_OPCODE_HANDLER_ARGS)
+{
+	if (XG(do_code_coverage)) {
+		zend_op *cur_opcode;
+		int      lineno;
+		char    *file;
+
+		zend_op_array *op_array = execute_data->op_array;
+
+		cur_opcode = *EG(opline_ptr);
+		lineno = cur_opcode->lineno;
+
+		file = op_array->filename;
+
+		xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
+	}
+	return ZEND_USER_OPCODE_DISPATCH;
+}
+
+static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
+{
+	zend_op       *cur_opcode, *next_opcode, *prev_opcode = NULL, *opcode_ptr;
+	zval          *dimval;
+	int            is_var, cv_len;
+	zend_op_array *op_array = execute_data->op_array;
+	xdebug_str     name = {0, 0, NULL};
+	int            gohungfound = 0, is_static = 0;
+	char          *zval_value = NULL;
+	xdebug_var_export_options *options;
+
+	cur_opcode = *EG(opline_ptr);
+	next_opcode = cur_opcode + 1;
+	prev_opcode = cur_opcode - 1;
+
+	if (cur_opcode->op1.op_type == IS_VAR &&
+			(next_opcode->op1.op_type == IS_VAR || cur_opcode->op2.op_type == IS_VAR) &&
+			prev_opcode->opcode == ZEND_FETCH_RW &&
+			prev_opcode->op1.op_type == IS_CONST &&
+			prev_opcode->op1.u.constant.type == IS_STRING
+	) {
+		xdebug_str_add(&name, xdebug_sprintf("$%s", prev_opcode->op1.u.constant.value.str.val), 1);
+	}
+
+	is_static = (prev_opcode->op1.op_type == IS_CONST && prev_opcode->op2.u.EA.type == ZEND_FETCH_STATIC_MEMBER);
+	options = xdebug_var_export_options_from_ini(TSRMLS_C);
+	options->no_decoration = 1;
+
+	if (cur_opcode->op1.op_type == IS_CV) {
+		xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, cur_opcode->op1.u.var, &cv_len)), 1);
+	} else if (cur_opcode->op1.op_type == IS_VAR && cur_opcode->opcode == ZEND_ASSIGN && prev_opcode->opcode == ZEND_FETCH_W) {
+		if (is_static) {
+			xdebug_str_add(&name, xdebug_sprintf("self::"), 1);
+		} else {
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &prev_opcode->op1, execute_data->Ts, &is_var), 0, options);
+			xdebug_str_add(&name, xdebug_sprintf("$%s", zval_value), 1);
+		}
+	} else if (is_static) { // todo : see if you can change this and the previous cases around
+		xdebug_str_add(&name, xdebug_sprintf("self::"), 1 );
+	}
+	if (cur_opcode->opcode >= ZEND_ASSIGN_ADD && cur_opcode->opcode <= ZEND_ASSIGN_BW_XOR ) {
+		if (cur_opcode->extended_value == ZEND_ASSIGN_OBJ) {
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &cur_opcode->op2, execute_data->Ts, &is_var), 0, options);
+			if (cur_opcode->op1.op_type == IS_UNUSED) {
+				xdebug_str_add(&name, xdebug_sprintf("$this->%s", zval_value), 1);
+			} else {
+				xdebug_str_add(&name, xdebug_sprintf("->%s", zval_value), 1);
+			}
+		} else if (cur_opcode->extended_value == ZEND_ASSIGN_DIM) {
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &cur_opcode->op2, execute_data->Ts, &is_var), 0, NULL);
+			xdebug_str_add(&name,xdebug_sprintf("[%s]", zval_value), 1);
+		}
+	}
+	if (zval_value) {
+		xdfree(zval_value);
+		zval_value = NULL;
+	}
+
+	/* Scroll back to start of FETCHES */
+	gohungfound = 0;
+	opcode_ptr = prev_opcode;
+	while (opcode_ptr->opcode == ZEND_FETCH_DIM_W || opcode_ptr->opcode == ZEND_FETCH_OBJ_W || opcode_ptr->opcode == ZEND_FETCH_W) {
+		opcode_ptr = opcode_ptr - 1;
+		gohungfound = 1;
+	}
+	opcode_ptr = opcode_ptr + 1;
+
+	if (gohungfound) {
+		do
+		{
+			if (opcode_ptr->op1.op_type == IS_UNUSED && opcode_ptr->opcode == ZEND_FETCH_OBJ_W) {
+				xdebug_str_add(&name, "$this", 0);
+			}
+			if (opcode_ptr->op1.op_type == IS_CV) {
+				xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, opcode_ptr->op1.u.var, &cv_len)), 1);
+			}
+			if (opcode_ptr->opcode == ZEND_FETCH_W) {
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &opcode_ptr->op1, execute_data->Ts, &is_var), 0, options);
+				xdebug_str_add(&name, xdebug_sprintf("%s", zval_value), 1);
+			}
+			if (opcode_ptr->opcode == ZEND_FETCH_DIM_W) {
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &opcode_ptr->op2, execute_data->Ts, &is_var), 0, NULL);
+				xdebug_str_add(&name, xdebug_sprintf("[%s]", zval_value), 1);
+			} else if (opcode_ptr->opcode == ZEND_FETCH_OBJ_W) {
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &opcode_ptr->op2, execute_data->Ts, &is_var), 0, options);
+				xdebug_str_add(&name, xdebug_sprintf("->%s", zval_value), 1);
+			}
+			opcode_ptr = opcode_ptr + 1;
+			if (zval_value) {
+				xdfree(zval_value);
+				zval_value = NULL;
+			}
+		} while (opcode_ptr->opcode == ZEND_FETCH_DIM_W || opcode_ptr->opcode == ZEND_FETCH_OBJ_W || opcode_ptr->opcode == ZEND_FETCH_W);
+	}
+
+	if (cur_opcode->opcode == ZEND_ASSIGN_OBJ) {
+		if (cur_opcode->op1.op_type == IS_UNUSED) {
+			xdebug_str_add(&name, "$this", 0);
+		}
+		dimval = xdebug_get_zval(execute_data, &cur_opcode->op2, execute_data->Ts, &is_var);
+		xdebug_str_add(&name, xdebug_sprintf("->%s", Z_STRVAL_P(dimval)), 1);
+	}
+
+	if (cur_opcode->opcode == ZEND_ASSIGN_DIM) {
+		if (next_opcode->opcode == ZEND_OP_DATA && cur_opcode->op2.op_type == IS_UNUSED) {
+			xdebug_str_add(&name, "[]", 0);
+		} else {
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, &opcode_ptr->op2, execute_data->Ts, &is_var), 0, NULL);
+			xdebug_str_add(&name, xdebug_sprintf("[%s]", zval_value), 1);
+			xdfree(zval_value);
+		}
+	}
+
+	xdfree(options->runtime);
+	xdfree(options);
+
+	return name.d;
+}
+
+static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_OPCODE_HANDLER_ARGS)
+{
+	char    *file;
+	zend_op_array *op_array = execute_data->op_array;
+	int            lineno;
+	zend_op       *cur_opcode, *next_opcode;
+	char          *full_varname;
+	zval          *val = NULL;
+	char          *t;
+	int            is_var;
+	function_stack_entry *fse;
+
+	cur_opcode = *EG(opline_ptr);
+	next_opcode = cur_opcode + 1;
+	file = op_array->filename;
+	lineno = cur_opcode->lineno;
+
+	if (do_cc && XG(do_code_coverage)) {
+		xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
+	}
+	if (XG(do_trace) && XG(trace_file) && XG(collect_assignments)) {
+		full_varname = xdebug_find_var_name(execute_data TSRMLS_CC);
+
+		if (cur_opcode->opcode >= ZEND_PRE_INC && cur_opcode->opcode <= ZEND_POST_DEC) {
+			char *tmp_varname;
+
+			switch (cur_opcode->opcode) {
+				case ZEND_PRE_INC:  tmp_varname = xdebug_sprintf("++%s", full_varname); break;
+				case ZEND_POST_INC: tmp_varname = xdebug_sprintf("%s++", full_varname); break;
+				case ZEND_PRE_DEC:  tmp_varname = xdebug_sprintf("--%s", full_varname); break;
+				case ZEND_POST_DEC: tmp_varname = xdebug_sprintf("%s--", full_varname); break;
+			}
+			xdfree(full_varname);
+			full_varname = tmp_varname;
+
+			val = xdebug_get_zval(execute_data, &cur_opcode->op1, execute_data->Ts, &is_var);
+		} else if (next_opcode->opcode == ZEND_OP_DATA) {
+			val = xdebug_get_zval(execute_data, &next_opcode->op1, execute_data->Ts, &is_var);
+		} else {
+			val = xdebug_get_zval(execute_data, &cur_opcode->op2, execute_data->Ts, &is_var);
+		}
+
+		fse = XDEBUG_LLIST_VALP(XDEBUG_LLIST_TAIL(XG(stack)));
+		t = xdebug_return_trace_assignment(fse, full_varname, val, op, file, lineno TSRMLS_CC);
+		xdfree(full_varname);
+		fprintf(XG(trace_file), "%s", t);
+		fflush(XG(trace_file));
+		xdfree(t);
+	}
+	return ZEND_USER_OPCODE_DISPATCH;
+}
+
+#define XDEBUG_OPCODE_OVERRIDE_ASSIGN(f,o,cc) \
+	int xdebug_##f##_handler(ZEND_OPCODE_HANDLER_ARGS) \
+	{ \
+		return xdebug_common_assign_dim_handler((o), (cc), ZEND_OPCODE_HANDLER_ARGS_PASSTHRU); \
+	}
+
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign,"=",1)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_add,"+=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_sub,"-=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_mul,"*=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_div,"/=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_mod,"%=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_sl,"<<=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_sr,">>=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(pre_inc,"",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(post_inc,"",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(pre_dec,"",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(post_dec,"",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_concat,".=",1)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_bw_or,"|=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_bw_and,"&=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_bw_xor,"^=",0)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_dim,"=",1)
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_obj,"=",1)
+
 void xdebug_count_line(char *filename, int lineno, int executable, int deadcode TSRMLS_DC)
 {
 	xdebug_coverage_file *file;
@@ -90,11 +313,10 @@ static void prefill_from_opcode(char *fn, zend_op opcode, int deadcode TSRMLS_DC
 		opcode.opcode != ZEND_EXT_NOP &&
 		opcode.opcode != ZEND_RECV &&
 		opcode.opcode != ZEND_RECV_INIT
-#ifdef ZEND_ENGINE_2
 		&& opcode.opcode != ZEND_VERIFY_ABSTRACT_CLASS
 		&& opcode.opcode != ZEND_OP_DATA
 		&& opcode.opcode != ZEND_ADD_INTERFACE
-#endif
+		&& opcode.opcode != ZEND_TICKS
 	) {
 		xdebug_count_line(fn, opcode.lineno, 1, deadcode TSRMLS_CC);
 	}
@@ -120,11 +342,7 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 
 	zend_op opcode = opa->opcodes[position];
 	if (opcode.opcode == ZEND_JMP) {
-#ifdef ZEND_ENGINE_2
 		*jmp1 = ((long) opcode.op1.u.jmp_addr - (long) base_address) / sizeof(zend_op);
-#else
-		*jmp1 = opcode.op1.u.opline_num;
-#endif
 		return 1;
 	} else if (
 		opcode.opcode == ZEND_JMPZ ||
@@ -133,11 +351,7 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 		opcode.opcode == ZEND_JMPNZ_EX
 	) {
 		*jmp1 = position + 1;
-#ifdef ZEND_ENGINE_2
 		*jmp2 = ((long) opcode.op2.u.jmp_addr - (long) base_address) / sizeof(zend_op);
-#else
-		*jmp2 = opcode.op1.u.opline_num;
-#endif
 		return 1;
 	} else if (opcode.opcode == ZEND_JMPZNZ) {
 		*jmp1 = opcode.op2.u.opline_num;
@@ -147,11 +361,7 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 		zend_brk_cont_element *el;
 
 		if (opcode.op2.op_type == IS_CONST
-#ifdef ZEND_ENGINE_2
 		    && opcode.op1.u.jmp_addr != (zend_op*) 0xFFFFFFFF
-#else
-		    && opcode.op1.u.opline_num > -1
-#endif
 		) {
 			el = xdebug_find_brk_cont(&opcode.op2.u.constant, opcode.op1.u.opline_num, opa);
 			*jmp1 = opcode.opcode == ZEND_BRK ? el->brk : el->cont;
@@ -161,14 +371,19 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 		*jmp1 = position + 1;
 		*jmp2 = opcode.op2.u.opline_num;
 		return 1;
+#if (PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3)
+	} else if (opcode.opcode == ZEND_GOTO) {
+		*jmp1 = ((long) opcode.op1.u.jmp_addr - (long) base_address) / sizeof(zend_op);
+		return 1;
+#endif
 	}
 	return 0;
 }
 
-static void xdebug_analyse_branch(zend_op_array *opa, unsigned int position, xdebug_set *set)
+static void xdebug_analyse_branch(zend_op_array *opa, unsigned int position, xdebug_set *set TSRMLS_DC)
 {
-	long jump_pos1 = -1;
-	long jump_pos2 = -1;
+	long jump_pos1;
+	long jump_pos2;
 
 	/*(fprintf(stderr, "Branch analysis from position: %d\n", position);)*/
 	/* First we see if the branch has been visited, if so we bail out. */
@@ -179,39 +394,30 @@ static void xdebug_analyse_branch(zend_op_array *opa, unsigned int position, xde
 	xdebug_set_add(set, position);
 	/*(fprintf(stderr, "XDEBUG Adding %d\n", position);)*/
 	while (position < opa->last) {
+		jump_pos1 = -1;
+		jump_pos2 = -1;
 
 		/* See if we have a jump instruction */
 		if (xdebug_find_jump(opa, position, &jump_pos1, &jump_pos2)) {
 			/*(fprintf(stderr, "XDEBUG Jump found. Position 1 = %d", jump_pos1);)*/
 			if (jump_pos2 != -1) {
-				/*(fprintf(stderr, ", Position 2 = %d\n", jump_pos2);)*/
+				/*(fprintf(stderr, ", Position 2 = %d\n", jump_pos2))*/;
 			} else {
-				/*(fprintf(stderr, "\n");)*/
+				/*(fprintf(stderr, "\n"))*/;
 			}
-			xdebug_analyse_branch(opa, jump_pos1, set);
+			xdebug_analyse_branch(opa, jump_pos1, set TSRMLS_CC);
 			if (jump_pos2 != -1 && jump_pos2 <= opa->last) {
-				xdebug_analyse_branch(opa, jump_pos2, set);
+				xdebug_analyse_branch(opa, jump_pos2, set TSRMLS_CC);
 			}
 			break;
 		}
-#ifdef ZEND_ENGINE_2
+
 		/* See if we have a throw instruction */
 		if (opa->opcodes[position].opcode == ZEND_THROW) {
 			/* fprintf(stderr, "Throw found at %d\n", position); */
-			/* Now we need to go forward to the first
-			 * zend_fetch_class/zend_catch combo */
-			while (position < opa->last) {
-				if (opa->opcodes[position].opcode == ZEND_CATCH) {
-					/* fprintf(stderr, "Found catch at %d\n", position); */
-					position--;
-					break;
-				}
-				position++;
-				/* fprintf(stderr, "Skipping %d\n", position); */
-			}
-			position--;
+			break;
 		}
-#endif
+
 		/* See if we have an exit instruction */
 		if (opa->opcodes[position].opcode == ZEND_EXIT) {
 			/* fprintf(stderr, "X* Return found\n"); */
@@ -229,17 +435,27 @@ static void xdebug_analyse_branch(zend_op_array *opa, unsigned int position, xde
 	}
 }
 
+static void xdebug_analyse_oparray(zend_op_array *opa, xdebug_set *set TSRMLS_DC)
+{
+	unsigned int position = 0;
+
+	while (position < opa->last) {
+		if (position == 0) {
+			xdebug_analyse_branch(opa, position, set TSRMLS_CC);
+		} else if (opa->opcodes[position].opcode == ZEND_CATCH) {
+			xdebug_analyse_branch(opa, position, set TSRMLS_CC);
+		}
+		position++;
+	}
+}
+
 static void prefill_from_oparray(char *fn, zend_op_array *opa TSRMLS_DC)
 {
-	char cache_key[256];
-	int  cache_key_len;
-	void *dummy;
 	unsigned int i;
 	xdebug_set *set = NULL;
 
 	opa->reserved[XG(reserved_offset)] = (void*) 1;
 
-#ifdef ZEND_ENGINE_2
 	/* Check for abstract methods and simply return from this function in those
 	 * cases. */
 #if PHP_VERSION_ID >= 50300
@@ -250,12 +466,11 @@ static void prefill_from_oparray(char *fn, zend_op_array *opa TSRMLS_DC)
 	{
 		return;
 	}	
-#endif
 
 	/* Run dead code analysis if requested */
 	if (XG(code_coverage_dead_code_analysis) && opa->done_pass_two) {
 		set = xdebug_set_create(opa->last);
-		xdebug_analyse_branch(opa, 0, set);
+		xdebug_analyse_oparray(opa, set TSRMLS_CC);
 	}
 
 	/* The normal loop then finally */
@@ -286,31 +501,19 @@ static int prefill_from_function_table(zend_op_array *opa XDEBUG_ZEND_HASH_APPLY
 	return ZEND_HASH_APPLY_KEEP;
 }
 
-#ifdef ZEND_ENGINE_2
 static int prefill_from_class_table(zend_class_entry **class_entry XDEBUG_ZEND_HASH_APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
-#else
-static int prefill_from_class_table(zend_class_entry *class_entry XDEBUG_ZEND_HASH_APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
-#endif
 {
 	char *new_filename;
 	zend_class_entry *ce;
 
-#ifdef ZEND_ENGINE_2
 	ce = *class_entry;
-#else
-	ce = class_entry;
-#endif
 
 	new_filename = va_arg(args, char*);
 	if (ce->type == ZEND_USER_CLASS) {
-#if PHP_MAJOR_VERSION >= 5
 		if (!(ce->ce_flags & ZEND_XDEBUG_VISITED)) {
 			ce->ce_flags |= ZEND_XDEBUG_VISITED;
 			zend_hash_apply_with_arguments(&ce->function_table XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) prefill_from_function_table, 1, new_filename);
 		}
-#else
-		zend_hash_apply_with_arguments(&ce->function_table, (apply_func_args_t) prefill_from_function_table, 1, new_filename);
-#endif
 	}
 
 	return ZEND_HASH_APPLY_KEEP;
@@ -337,9 +540,11 @@ PHP_FUNCTION(xdebug_start_code_coverage)
 	XG(code_coverage_dead_code_analysis) = (options & XDEBUG_CC_OPTION_DEAD_CODE);
 
 	if (XG(extended_info)) {
+		RETVAL_BOOL(!XG(do_code_coverage));
 		XG(do_code_coverage) = 1;
 	} else {
 		php_error(E_WARNING, "You can only use code coverage when you leave the setting of 'xdebug.extended_info' to the default '1'.");
+		RETVAL_BOOL(0);
 	}
 }
 
@@ -356,7 +561,9 @@ PHP_FUNCTION(xdebug_stop_code_coverage)
 			XG(code_coverage) = xdebug_hash_alloc(32, xdebug_coverage_file_dtor);
 		}
 		XG(do_code_coverage) = 0;
+		RETURN_TRUE;
 	}
+	RETURN_FALSE;
 }
 
 
