@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007 Derick Rethans      |
+   | Copyright (c) 2002-2010 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -21,30 +21,22 @@
 
 #include "php.h"
 
-#if ((PHP_MAJOR_VERSION == 4) && (PHP_MINOR_VERSION == 3) && (PHP_RELEASE_VERSION <= 1)) || PHP_MAJOR_VERSION >= 6
-
+#if PHP_MAJOR_VERSION >= 6
 void xdebug_php_var_dump(zval **struc, int level TSRMLS_DC);
-
 #else
+# include "ext/standard/php_var.h"
+# define xdebug_php_var_dump php_var_dump
+#endif
 
-#	include "ext/standard/php_var.h"
-#	define xdebug_php_var_dump php_var_dump
-
+#if PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION == 1
+# define zend_memrchr php_zend_memrchr
+void *php_zend_memrchr(const void *s, int c, size_t n);
 #endif
 
 
-#if (PHP_MAJOR_VERSION == 4) && (PHP_MINOR_VERSION == 3) && (PHP_RELEASE_VERSION <= 4)
-
-unsigned char *xdebug_base64_encode(const unsigned char *, int, int *);
-unsigned char *xdebug_base64_decode(const unsigned char *str, int length, int *ret_length);
-
-#else
-
-#	include "ext/standard/base64.h"
-#	define xdebug_base64_encode php_base64_encode
-#	define xdebug_base64_decode php_base64_decode
-
-#endif
+#include "ext/standard/base64.h"
+#define xdebug_base64_encode php_base64_encode
+#define xdebug_base64_decode php_base64_decode
 
 zval *xdebug_zval_ptr(znode *node, temp_variable *Ts TSRMLS_DC);
 
