@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007 Derick Rethans      |
+   | Copyright (c) 2002-2010 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -33,6 +33,7 @@
 #include "php_globals.h"
 #include "php_xdebug.h"
 #include "xdebug_private.h"
+#include "xdebug_code_coverage.h"
 #include "xdebug_com.h"
 #include "xdebug_handler_dbgp.h"
 #include "xdebug_hash.h"
@@ -167,6 +168,7 @@ DBGP_FUNC(detach);
 
 /* Non standard comments */
 DBGP_FUNC(xcmd_profiler_name_get);
+DBGP_FUNC(xcmd_get_executable_lines);
 
 /*****************************************************************************
 ** Dispatcher tables for supported debug commands
@@ -209,7 +211,8 @@ static xdebug_dbgp_cmd dbgp_commands[] = {
 	DBGP_STOP_FUNC_ENTRY(detach,       XDEBUG_DBGP_NONE)
 
 	/* Non standard functions */
-	DBGP_FUNC_ENTRY(xcmd_profiler_name_get, XDEBUG_DBGP_POST_MORTEM)
+	DBGP_FUNC_ENTRY(xcmd_profiler_name_get,    XDEBUG_DBGP_POST_MORTEM)
+	DBGP_FUNC_ENTRY(xcmd_get_executable_lines, XDEBUG_DBGP_NONE)
 	{ NULL, NULL }
 };
 
@@ -271,6 +274,45 @@ static void send_message(xdebug_con *context, xdebug_xml_node *message TSRMLS_DC
 #define XF_ST_ARRAY_INDEX_ASSOC  2
 #define XF_ST_OBJ_PROPERTY       3
 
+inline static HashTable *fetch_ht_from_zval(zval *z TSRMLS_DC)
+{
+	switch (Z_TYPE_P(z)) {
+		case IS_ARRAY:
+			return Z_ARRVAL_P(z);
+			break;
+		case IS_OBJECT:
+			return Z_OBJPROP_P(z);
+			break;
+	}
+	return NULL;
+}
+
+inline static char *fetch_classname_from_zval(zval *z, int *length TSRMLS_DC)
+{
+	char *name;
+	zend_uint name_len;
+
+	if (Z_TYPE_P(z) != IS_OBJECT) {
+		return NULL;
+	}
+
+	if (Z_OBJ_HT_P(z)->get_class_name == NULL ||
+		Z_OBJ_HT_P(z)->get_class_name(z, &name, &name_len, 0 TSRMLS_CC) != SUCCESS) {
+		zend_class_entry *ce;
+
+		ce = zend_get_class_entry(z TSRMLS_CC);
+		if (!ce) {
+			return NULL;
+		}
+
+		*length = ce->name_length;
+		return estrdup(ce->name);
+	} 
+
+	*length = name_len;
+	return name;
+}
+
 static char* prepare_search_key(char *name, int *name_length, char *prefix, int prefix_length)
 {
 	char *element;
@@ -303,19 +345,47 @@ static zval* fetch_zval_from_symbol_table(HashTable *ht, char* name, int name_le
 
 	switch (type) {
 		case XF_ST_ROOT:
+#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
+			/* Check for compiled vars */
+			element = prepare_search_key(name, &element_length, "", 0);
+			if (XG(active_execute_data) && XG(active_op_array)) {
+				int i = 0;
+				ulong hash_value = zend_inline_hash_func(element, element_length + 1);
+				zend_op_array *opa = XG(active_op_array);
+				zval ***CVs = XG(active_execute_data)->CVs;
+
+				while (i < opa->last_var) {
+					if (opa->vars[i].hash_value == hash_value &&
+						opa->vars[i].name_len == element_length &&
+						strcmp(opa->vars[i].name, element) == 0)
+					{
+						if (CVs[i]) {
+							retval_p = *CVs[i];
+							goto cleanup;
+						}
+					}
+					i++;
+				}
+			}
+			free(element);
+			ht = XG(active_symbol_table);
+#else
+			ht = XG(active_symbol_table);
+			/* break intentionally missing */
+#endif
 		case XF_ST_ARRAY_INDEX_ASSOC:
 			element = prepare_search_key(name, &name_length, "", 0);
-#ifdef ZEND_ENGINE_2
+
 			/* Handle "this" in a different way */
 			if (type == XF_ST_ROOT && strcmp("this", element) == 0) {
-				if (XG(active_execute_data)) {
-					retval_p = XG(active_execute_data)->object;
+				if (XG(This)) {
+					retval_p = XG(This);
 				} else {
 					retval_p = NULL;
 				}
 				goto cleanup;
 			}
-#endif
+
 			if (ht && zend_hash_find(ht, element, name_length + 1, (void **) &retval_pp) == SUCCESS) {
 				retval_p = *retval_pp;
 				goto cleanup;
@@ -325,7 +395,7 @@ static zval* fetch_zval_from_symbol_table(HashTable *ht, char* name, int name_le
 			element = prepare_search_key(name, &name_length, "", 0);
 			if (ht && zend_hash_index_find(ht, strtoul(element, NULL, 10), (void **) &retval_pp) == SUCCESS) {
 				retval_p = *retval_pp;
-				goto cleanup_num;
+				goto cleanup;
 			}
 			break;
 		case XF_ST_OBJ_PROPERTY:
@@ -357,219 +427,138 @@ static zval* fetch_zval_from_symbol_table(HashTable *ht, char* name, int name_le
 	}
 cleanup:
 	free(element);
-cleanup_num:
 	return retval_p;
 }
 
-inline static HashTable *fetch_ht_from_zval(zval *z TSRMLS_DC)
-{
-	switch (Z_TYPE_P(z)) {
-		case IS_ARRAY:
-			return Z_ARRVAL_P(z);
-			break;
-		case IS_OBJECT:
-			return Z_OBJPROP_P(z);
-			break;
-	}
-	return NULL;
-}
-
-inline static char *fetch_classname_from_zval(zval *z, int *length TSRMLS_DC)
-{
-	if (Z_TYPE_P(z) != IS_OBJECT) {
-		return NULL;
-	}
-#if PHP_MAJOR_VERSION == 4
-	{
-		zend_class_entry *ce;
-		ce = Z_OBJCE_P(z);
-		*length = ce->name_length;
-		return estrdup(ce->name);
-	}
-#endif
-#if PHP_MAJOR_VERSION >= 5
-	{
-		char *name;
-		zend_uint name_len;
-
-		if (Z_OBJ_HT_P(z)->get_class_name == NULL ||
-			Z_OBJ_HT_P(z)->get_class_name(z, &name, &name_len, 0 TSRMLS_CC) != SUCCESS) {
-			zend_class_entry *ce;
-
-			ce = zend_get_class_entry(z TSRMLS_CC);
-			if (!ce) {
-				return NULL;
-			}
-
-			*length = ce->name_length;
-			return estrdup(ce->name);
-		} 
-
-		*length = name_len;
-		return name;
-	}
-#endif
-	return NULL;
-}
-
 static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 {
-	HashTable           *st = NULL;
+	HashTable *st = NULL;
+	int        found = -1;
+	int        state = 0;
+	char     **p = &name;
+	char      *keyword = NULL, *keyword_end = NULL;
+	int        type = XF_ST_ROOT;
+	zval      *retval = NULL;
+	char      *current_classname = NULL;
+	int        cc_length = 0;
+	char       quotechar = 0;
 
-	if (name[0] == '$') {
-		/* This is a fullname property. Relying on eval is bad as it can bail
-		 * out, so we do it ourselves. */
-		int found = -1;
-		int state = 0;
-		char **p = &name;
-		char *keyword = NULL, *keyword_end = NULL;
-		int type = XF_ST_ROOT;
-		zval *retval = NULL;
-		char *current_classname = NULL;
-		int   cc_length = 0;
-		char  quotechar = 0;
-
-		/* Set the target table to the currently active scope */
-		st = XG(active_symbol_table);
-
-		do {
-			if (*p[0] == '\0') {
-				found = 0;
-			} else {
-				switch (state) {
-					case 0:
-						if (*p[0] == '$') {
-							state = 1;
-							keyword = *p + 1;
-							break;
+	do {
+		if (*p[0] == '\0') {
+			found = 0;
+		} else {
+			switch (state) {
+				case 0:
+					if (*p[0] == '$') {
+						keyword = *p + 1;
+						break;
+					}
+					keyword = *p;
+					state = 1;
+					/* break intentionally missing */
+				case 1:
+					if (*p[0] == '[') {
+						keyword_end = *p;
+						if (keyword) {
+							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+							if (current_classname) {
+								efree(current_classname);
+							}
+							current_classname = NULL;
+							if (retval) {
+								st = fetch_ht_from_zval(retval TSRMLS_CC);
+							}
+							keyword = NULL;
 						}
+						state = 3;
+					} else if (*p[0] == '-') {
+						keyword_end = *p;
+						if (keyword) {
+							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+							if (current_classname) {
+								efree(current_classname);
+							}
+							current_classname = NULL;
+							if (retval) {
+								current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
+								st = fetch_ht_from_zval(retval TSRMLS_CC);
+							}
+							keyword = NULL;
+						}
+						state = 2;
+						type = XF_ST_OBJ_PROPERTY;
+					}
+					break;
+				case 2:
+					if (*p[0] != '>') {
 						keyword = *p;
-						/* break intentionally missing */
-					case 1:
-						if (*p[0] == '[') {
-							keyword_end = *p;
-							if (keyword) {
-								retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
-								if (current_classname) {
-									efree(current_classname);
-								}
-								current_classname = NULL;
-								if (retval) {
-									st = fetch_ht_from_zval(retval TSRMLS_CC);
-								}
-								keyword = NULL;
-							}
-							state = 3;
-						} else if (*p[0] == '-') {
-							keyword_end = *p;
-							if (keyword) {
-								retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
-								if (current_classname) {
-									efree(current_classname);
-								}
-								current_classname = NULL;
-								if (retval) {
-									current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
-									st = fetch_ht_from_zval(retval TSRMLS_CC);
-								}
-								keyword = NULL;
-							}
-							state = 2;
-							type = XF_ST_OBJ_PROPERTY;
+						state = 1;
+					}
+					break;
+				case 3:
+					/* Associative arrays */
+					if (*p[0] == '\'' || *p[0] == '"') {
+						state = 4;
+						keyword = *p + 1;
+						quotechar = *p[0];
+						type = XF_ST_ARRAY_INDEX_ASSOC;
+					}
+					/* Numerical index */
+					if (*p[0] >= '0' && *p[0] <= '9') {
+						state = 6;
+						keyword = *p;
+						type = XF_ST_ARRAY_INDEX_NUM;
+					}
+					break;
+				case 4:
+					if (*p[0] == quotechar) {
+						quotechar = 0;
+						state = 5;
+						keyword_end = *p;
+						retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+						if (current_classname) {
+							efree(current_classname);
 						}
-						break;
-					case 2:
-						if (*p[0] != '>') {
-							keyword = *p;
-							state = 1;
+						current_classname = NULL;
+						if (retval) {
+							current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
+							st = fetch_ht_from_zval(retval TSRMLS_CC);
 						}
-						break;
-					case 3:
-						/* Associative arrays */
-						if (*p[0] == '\'' || *p[0] == '"') {
-							state = 4;
-							keyword = *p + 1;
-							quotechar = *p[0];
-							type = XF_ST_ARRAY_INDEX_ASSOC;
+						keyword = NULL;
+					}
+					break;
+				case 5:
+					if (*p[0] == ']') {
+						state = 1;
+					}
+					break;
+				case 6:
+					if (*p[0] == ']') {
+						state = 1;
+						keyword_end = *p;
+						retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+						if (current_classname) {
+							efree(current_classname);
 						}
-						/* Numerical index */
-						if (*p[0] >= '0' && *p[0] <= '9') {
-							state = 6;
-							keyword = *p;
-							type = XF_ST_ARRAY_INDEX_NUM;
+						current_classname = NULL;
+						if (retval) {
+							current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
+							st = fetch_ht_from_zval(retval TSRMLS_CC);
 						}
-						break;
-					case 4:
-						if (*p[0] == quotechar) {
-							quotechar = 0;
-							state = 5;
-							keyword_end = *p;
-							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
-							if (current_classname) {
-								efree(current_classname);
-							}
-							current_classname = NULL;
-							if (retval) {
-								current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
-								st = fetch_ht_from_zval(retval TSRMLS_CC);
-							}
-							keyword = NULL;
-						}
-						break;
-					case 5:
-						if (*p[0] == ']') {
-							state = 1;
-						}
-						break;
-					case 6:
-						if (*p[0] == ']') {
-							state = 1;
-							keyword_end = *p;
-							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
-							if (current_classname) {
-								efree(current_classname);
-							}
-							current_classname = NULL;
-							if (retval) {
-								current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
-								st = fetch_ht_from_zval(retval TSRMLS_CC);
-							}
-							keyword = NULL;
-						}
-						break;
-				}
-				(*p)++;
-			}
-		} while (found < 0);
-		if (keyword != NULL) {
-			retval = fetch_zval_from_symbol_table(st, keyword, *p - keyword, type, current_classname, cc_length TSRMLS_CC);
-			if (retval) {
-				st = fetch_ht_from_zval(retval TSRMLS_CC);
+						keyword = NULL;
+					}
+					break;
 			}
+			(*p)++;
 		}
-		return retval;
-	} else {
-		zval **retval;
-
-		st = XG(active_symbol_table);
-		if (st && zend_hash_find(st, name, name_length, (void **) &retval) == SUCCESS) {
-			return *retval;
+	} while (found < 0);
+	if (keyword != NULL) {
+		retval = fetch_zval_from_symbol_table(st, keyword, *p - keyword, type, current_classname, cc_length TSRMLS_CC);
+		if (retval) {
+			st = fetch_ht_from_zval(retval TSRMLS_CC);
 		}
-
-		st = EG(active_op_array)->static_variables;
-		if (st) {
-			if (zend_hash_find(st, name, name_length, (void **) &retval) == SUCCESS) {
-				return *retval;
-			}
-		}
-#if 0
-		st = &EG(symbol_table);
-		if (zend_hash_find(st, name, name_length, (void **) &retval) == SUCCESS) {
-			return *retval;
-		}
-#endif
 	}
-	return NULL;
+	return retval;
 }
 
 static xdebug_xml_node* get_symbol(char* name, int name_length, xdebug_var_export_options *options TSRMLS_DC)
@@ -833,6 +822,11 @@ static void breakpoint_brk_info_add(xdebug_xml_node *xml, xdebug_brk_info *brk)
 			xdebug_xml_add_attribute(xml, "hit_condition", "%");
 			break;
 	}
+	if (brk->condition) {
+		xdebug_xml_node *condition = xdebug_xml_node_init("expression");
+		xdebug_xml_add_text_ex(condition, brk->condition, strlen(brk->condition), 0, 1);
+		xdebug_xml_add_child(xml, condition);
+	}
 	xdebug_xml_add_attribute_ex(xml, "hit_value", xdebug_sprintf("%lu", brk->hit_value), 0, 1);
 }
 
@@ -871,12 +865,6 @@ static xdebug_brk_info* breakpoint_brk_info_fetch(int type, char *hkey)
 			}
 			break;
 
-		case BREAKPOINT_TYPE_METHOD:
-			if (xdebug_hash_find(XG(context).class_breakpoints, hkey, strlen(hkey), (void *) &brk)) {
-				return brk;
-			}
-			break;
-
 		case BREAKPOINT_TYPE_EXCEPTION:
 			if (xdebug_hash_find(XG(context).exception_breakpoints, hkey, strlen(hkey), (void *) &brk)) {
 				return brk;
@@ -891,6 +879,7 @@ static int breakpoint_remove(int type, char *hkey)
 	xdebug_llist_element *le;
 	xdebug_brk_info      *brk = NULL;
 	xdebug_arg           *parts = (xdebug_arg*) xdmalloc(sizeof(xdebug_arg));
+	int                   retval = FAILURE;
 	TSRMLS_FETCH();
 
 	switch (type) {
@@ -906,7 +895,8 @@ static int breakpoint_remove(int type, char *hkey)
 
 				if (atoi(parts->args[1]) == brk->lineno && memcmp(brk->file, parts->args[0], brk->file_len) == 0) {
 					xdebug_llist_remove(XG(context).line_breakpoints, le, NULL);
-					return SUCCESS;
+					retval = SUCCESS;
+					break;
 				}
 			}
 
@@ -916,23 +906,17 @@ static int breakpoint_remove(int type, char *hkey)
 
 		case BREAKPOINT_TYPE_FUNCTION:
 			if (xdebug_hash_delete(XG(context).function_breakpoints, hkey, strlen(hkey))) {
-				return SUCCESS;
-			}
-			break;
-
-		case BREAKPOINT_TYPE_METHOD:
-			if (xdebug_hash_delete(XG(context).class_breakpoints, hkey, strlen(hkey))) {
-				return SUCCESS;
+				retval = SUCCESS;
 			}
 			break;
 
 		case BREAKPOINT_TYPE_EXCEPTION:
 			if (xdebug_hash_delete(XG(context).exception_breakpoints, hkey, strlen(hkey))) {
-				return SUCCESS;
+				retval = SUCCESS;
 			}
 			break;
 	}
-	return FAILURE;
+	return retval;
 }
 
 #define BREAKPOINT_ACTION_GET       1
@@ -1096,8 +1080,11 @@ DBGP_FUNC(breakpoint_set)
 		BREAKPOINT_CHANGE_OPERATOR();
 		brk_info->hit_value = strtol(CMD_OPTION('h'), NULL, 10);
 	}
+	if (CMD_OPTION('r')) {
+		brk_info->temporary = strtol(CMD_OPTION('r'), NULL, 10);
+	}
 
-	if (strcmp(CMD_OPTION('t'), "line") == 0) {
+	if ((strcmp(CMD_OPTION('t'), "line") == 0) || (strcmp(CMD_OPTION('t'), "conditional") == 0)) {
 		if (!CMD_OPTION('n')) {
 			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_INVALID_ARGS);
 		}
@@ -1144,8 +1131,8 @@ DBGP_FUNC(breakpoint_set)
 
 			brk_info->classname = xdstrdup(CMD_OPTION('a'));
 			tmp_name = xdebug_sprintf("%s::%s", CMD_OPTION('a'), CMD_OPTION('m'));
-			res = xdebug_hash_add(context->class_breakpoints, tmp_name, strlen(tmp_name), (void*) brk_info);
-			brk_id = breakpoint_admin_add(context, BREAKPOINT_TYPE_METHOD, tmp_name);
+			res = xdebug_hash_add(context->function_breakpoints, tmp_name, strlen(tmp_name), (void*) brk_info);
+			brk_id = breakpoint_admin_add(context, BREAKPOINT_TYPE_FUNCTION, tmp_name);
 			xdfree(tmp_name);
 
 			if (!res) {
@@ -1160,12 +1147,7 @@ DBGP_FUNC(breakpoint_set)
 		}
 	} else
 
-	if (strcmp(CMD_OPTION('t'), "conditional") == 0) {
-		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_BREAKPOINT_TYPE_NOT_SUPPORTED);
-	} else
-
 	if (strcmp(CMD_OPTION('t'), "exception") == 0) {
-#if PHP_MAJOR_VERSION >= 5
 		if (!CMD_OPTION('x')) {
 			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_INVALID_ARGS);
 		}
@@ -1175,9 +1157,6 @@ DBGP_FUNC(breakpoint_set)
 		} else {
 			brk_id = breakpoint_admin_add(context, BREAKPOINT_TYPE_EXCEPTION, CMD_OPTION('x'));
 		}
-#else
-		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_BREAKPOINT_TYPE_NOT_SUPPORTED);
-#endif
 	} else
 
 	if (strcmp(CMD_OPTION('t'), "watch") == 0) {
@@ -1222,10 +1201,18 @@ DBGP_FUNC(eval)
 
 	options = (xdebug_var_export_options*) context->options;
 	
+	if (CMD_OPTION('p')) {
+		options->runtime[0].page = strtol(CMD_OPTION('p'), NULL, 10);
+	} else {
+		options->runtime[0].page = 0;
+	}
+
 	/* base64 decode eval string */
 	eval_string = (char*) xdebug_base64_decode((unsigned char*) CMD_OPTION('-'), strlen(CMD_OPTION('-')), &new_length);
 
-	res = xdebug_do_eval(eval_string, &ret_zval TSRMLS_CC);
+	zend_try {
+		res = xdebug_do_eval(eval_string, &ret_zval TSRMLS_CC);
+	} zend_end_try();
 
 	efree(eval_string);
 
@@ -1388,11 +1375,11 @@ DBGP_FUNC(detach)
 	XG(status) = DBGP_STATUS_DETACHED;
 	xdebug_xml_add_attribute(*retval, "status", xdebug_dbgp_status_strings[DBGP_STATUS_STOPPED]);
 	xdebug_xml_add_attribute(*retval, "reason", xdebug_dbgp_reason_strings[XG(reason)]);
+	XG(context).handler->remote_deinit(&(XG(context)));
 	XG(remote_enabled) = 0;
 	XG(stdout_redirected) = 0;
 	XG(stderr_redirected) = 0;
 	XG(stdin_redirected) = 0;
-	xdebug_close_socket(context->socket);
 }
 
 
@@ -1450,7 +1437,7 @@ DBGP_FUNC(feature_get)
 		XDEBUG_STR_CASE_END
 
 		XDEBUG_STR_CASE("breakpoint_types")
-			xdebug_xml_add_text(*retval, xdstrdup("line call return"));
+			xdebug_xml_add_text(*retval, xdstrdup("line conditional call return exception"));
 			xdebug_xml_add_attribute(*retval, "supported", "1");
 		XDEBUG_STR_CASE_END
 
@@ -1637,8 +1624,20 @@ DBGP_FUNC(property_get)
 	/* Set the symbol table corresponding with the requested stack depth */
 	if (context_nr == 0) { /* locals */
 		if ((fse = xdebug_get_stack_frame(depth TSRMLS_CC))) {
-			XG(active_symbol_table) = fse->symbol_table;
+#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
+			function_stack_entry *old_fse = xdebug_get_stack_frame(depth - 1 TSRMLS_CC);
+
+			if (depth > 0) {
+				XG(active_execute_data) = old_fse->execute_data;
+			} else {
+				XG(active_execute_data) = EG(current_execute_data);
+			}
+#else
 			XG(active_execute_data) = fse->execute_data;
+#endif
+			XG(active_symbol_table) = fse->symbol_table;
+			XG(active_op_array)     = fse->op_array;
+			XG(This)                = fse->This;
 		} else {
 			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_STACK_DEPTH_INVALID);
 		}
@@ -1661,6 +1660,7 @@ DBGP_FUNC(property_get)
 		options->max_data = old_max_data;
 		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTANT);
 	}
+	XG(active_op_array) = NULL;
 }
 
 DBGP_FUNC(property_set)
@@ -1697,8 +1697,20 @@ DBGP_FUNC(property_set)
 	/* Set the symbol table corresponding with the requested stack depth */
 	if (context_nr == 0) { /* locals */
 		if ((fse = xdebug_get_stack_frame(depth TSRMLS_CC))) {
-			XG(active_symbol_table) = fse->symbol_table;
+#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
+			function_stack_entry *old_fse = xdebug_get_stack_frame(depth - 1 TSRMLS_CC);
+
+			if (depth > 0) {
+				XG(active_execute_data) = old_fse->execute_data;
+			} else {
+				XG(active_execute_data) = EG(current_execute_data);
+			}
+#else
 			XG(active_execute_data) = fse->execute_data;
+#endif
+			XG(active_symbol_table) = fse->symbol_table;
+			XG(active_op_array)     = fse->op_array;
+			XG(This)                = fse->This;
 		} else {
 			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_STACK_DEPTH_INVALID);
 		}
@@ -1798,8 +1810,20 @@ DBGP_FUNC(property_value)
 
 	/* Set the symbol table corresponding with the requested stack depth */
 	if ((fse = xdebug_get_stack_frame(depth TSRMLS_CC))) {
-		XG(active_symbol_table) = fse->symbol_table;
+#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
+		function_stack_entry *old_fse = xdebug_get_stack_frame(depth - 1 TSRMLS_CC);
+
+		if (depth > 0) {
+			XG(active_execute_data) = old_fse->execute_data;
+		} else {
+			XG(active_execute_data) = EG(current_execute_data);
+		}
+#else
 		XG(active_execute_data) = fse->execute_data;
+#endif
+		XG(active_symbol_table) = fse->symbol_table;
+		XG(active_op_array)     = fse->op_array;
+		XG(This)                = fse->This;
 	} else {
 		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_STACK_DEPTH_INVALID);
 	}
@@ -1829,7 +1853,7 @@ static void attach_used_var_with_contents(void *xml, xdebug_hash_element* he, vo
 	xdebug_xml_node    *contents;
 	TSRMLS_FETCH();
 
-	contents = get_symbol(name, strlen(name) + 1, options TSRMLS_CC);
+	contents = get_symbol(name, strlen(name), options TSRMLS_CC);
 	if (contents) {
 		xdebug_xml_add_child(node, contents);
 	} else {
@@ -1840,7 +1864,7 @@ static void attach_used_var_with_contents(void *xml, xdebug_hash_element* he, vo
 			full_name = xdstrdup(name);
 		}
 		xdebug_xml_add_attribute_ex(contents, "name", xdstrdup(name), 0, 1);
-		xdebug_xml_add_attribute_ex(contents, "fullname", xdstrdup(full_name), 0, 1);
+		xdebug_xml_add_attribute_ex(contents, "fullname", full_name, 0, 1);
 
 		xdebug_xml_add_attribute(contents, "type", "uninitialized");
 		xdebug_xml_add_child(node, contents);
@@ -1850,6 +1874,7 @@ static void attach_used_var_with_contents(void *xml, xdebug_hash_element* he, vo
 static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options *options, long context_id, long depth, void (*func)(void *, xdebug_hash_element*, void*) TSRMLS_DC)
 {
 	function_stack_entry *fse;
+	char                 *var_name;
 
 	/* right now, we only have zero or one, one being globals, which is
 	 * always the head of the stack */
@@ -1864,30 +1889,46 @@ static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options
 		add_variable_node(node, "_REQUEST", sizeof("_REQUEST"), 1, 1, 0, options TSRMLS_CC);
 		add_variable_node(node, "_SERVER", sizeof("_SERVER"), 1, 1, 0, options TSRMLS_CC);
 		add_variable_node(node, "_SESSION", sizeof("_SESSION"), 1, 1, 0, options TSRMLS_CC);
+		add_variable_node(node, "GLOBALS", sizeof("GLOBALS"), 1, 1, 0, options TSRMLS_CC);
 		XG(active_symbol_table) = NULL;
 		return 0;
 	}
 
 	/* Here the context_id is 0 */
 	if ((fse = xdebug_get_stack_frame(depth TSRMLS_CC))) {
-		XG(active_symbol_table) = fse->symbol_table;
+#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
+		function_stack_entry *old_fse = xdebug_get_stack_frame(depth - 1 TSRMLS_CC);
+
+		if (depth > 0) {
+			XG(active_execute_data) = old_fse->execute_data;
+		} else {
+			XG(active_execute_data) = EG(current_execute_data);
+		}
+#else
 		XG(active_execute_data) = fse->execute_data;
+#endif
+		XG(active_symbol_table) = fse->symbol_table;
+		XG(active_op_array)     = fse->op_array;
+		XG(This)                = fse->This;
 
 		/* Only show vars when they are scanned */
 		if (fse->used_vars) {
 			xdebug_hash *tmp_hash;
 			tmp_hash = xdebug_used_var_hash_from_llist(fse->used_vars);
 			xdebug_hash_apply_with_argument(tmp_hash, (void *) node, func, (void *) options);
+
+			/* zend engine 2 does not give us $this, eval so we can get it */
+			if (!xdebug_hash_find(tmp_hash, "this", 4, (void *) &var_name)) {
+				add_variable_node(node, "this", sizeof("this"), 1, 1, 0, options TSRMLS_CC);
+			}
+
 			xdebug_hash_destroy(tmp_hash);
 		}
 
-#ifdef ZEND_ENGINE_2
-		/* zend engine 2 does not give us $this, eval so we can get it */
-		add_variable_node(node, "$this", sizeof("$this"), 1, 1, 0, options TSRMLS_CC);
-#endif
-
 		XG(active_symbol_table) = NULL;
 		XG(active_execute_data) = NULL;
+		XG(active_op_array)     = NULL;
+		XG(This)                = NULL;
 		return 0;
 	}
 	
@@ -1980,6 +2021,34 @@ DBGP_FUNC(xcmd_profiler_name_get)
 	}
 }
 
+DBGP_FUNC(xcmd_get_executable_lines)
+{
+	function_stack_entry *fse;
+	int                   i, depth;
+	xdebug_xml_node      *lines, *line;
+
+	if (!CMD_OPTION('d')) {
+		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_INVALID_ARGS);
+	}
+
+	depth = strtol(CMD_OPTION('d'), NULL, 10);
+	if (depth >= 0 && depth < XG(level)) {
+		fse = xdebug_get_stack_frame(depth TSRMLS_CC);
+	} else {
+		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_STACK_DEPTH_INVALID);
+	}
+
+	lines = xdebug_xml_node_init("xdebug:lines");
+	for (i = 0; i < fse->op_array->size; i++ ) {
+		if (fse->op_array->opcodes[i].opcode == ZEND_EXT_STMT ) {
+			line = xdebug_xml_node_init("xdebug:line");
+			xdebug_xml_add_attribute_ex(line, "lineno", xdebug_sprintf("%lu", fse->op_array->opcodes[i].lineno), 0, 1);
+			xdebug_xml_add_child(lines, line);
+		}
+	}
+	xdebug_xml_add_child(*retval, lines);
+}
+
 
 /*****************************************************************************
 ** Parsing functions
@@ -1999,7 +2068,7 @@ static void xdebug_dbgp_arg_dtor(xdebug_dbgp_arg *arg)
 {
 	int i;
 
-	for (i = 0; i < 26; i++) {
+	for (i = 0; i < 27; i++) {
 		if (arg->value[i]) {
 			xdfree(arg->value[i]);
 		}
@@ -2183,6 +2252,9 @@ static int xdebug_dbgp_parse_option(xdebug_con *context, char* line, int flags,
 				XG(reason) = DBGP_REASON_OK;
 			}
 			XG(lastcmd) = command->name;
+			if (XG(lasttransid)) {
+				xdfree(XG(lasttransid));
+			}
 			XG(lasttransid) = xdstrdup(CMD_OPTION('i'));
 			if (XG(status) != DBGP_STATUS_STOPPING || (XG(status) == DBGP_STATUS_STOPPING && command->flags & XDEBUG_DBGP_POST_MORTEM)) {
 				command->handler((xdebug_xml_node**) &retval, context, args TSRMLS_CC);
@@ -2216,7 +2288,7 @@ static int xdebug_dbgp_parse_option(xdebug_con *context, char* line, int flags,
 
 char *xdebug_dbgp_get_revision(void)
 {
-	return "$Revision: 1.125.2.6 $";
+	return "$Revision: 1.145 $";
 }
 
 static int xdebug_dbgp_cmdloop(xdebug_con *context, int bail TSRMLS_DC)
@@ -2345,23 +2417,9 @@ int xdebug_dbgp_init(xdebug_con *context, int mode)
 		options->runtime[i].current_element_nr = 0;
 	}
 
-/* {{{ Initialize auto globals in Zend Engine 2 */
-#ifdef ZEND_ENGINE_2
-	zend_is_auto_global("_ENV",     sizeof("_ENV")-1     TSRMLS_CC);
-	zend_is_auto_global("_GET",     sizeof("_GET")-1     TSRMLS_CC);
-	zend_is_auto_global("_POST",    sizeof("_POST")-1    TSRMLS_CC);
-	zend_is_auto_global("_COOKIE",  sizeof("_COOKIE")-1  TSRMLS_CC);
-	zend_is_auto_global("_REQUEST", sizeof("_REQUEST")-1 TSRMLS_CC);
-	zend_is_auto_global("_FILES",   sizeof("_FILES")-1   TSRMLS_CC);
-	zend_is_auto_global("_SERVER",  sizeof("_SERVER")-1  TSRMLS_CC);
-	zend_is_auto_global("_SESSION", sizeof("_SESSION")-1 TSRMLS_CC);
-#endif
-/* }}} */
-
 	context->breakpoint_list = xdebug_hash_alloc(64, (xdebug_hash_dtor) xdebug_hash_admin_dtor);
 	context->function_breakpoints = xdebug_hash_alloc(64, (xdebug_hash_dtor) xdebug_hash_brk_dtor);
 	context->exception_breakpoints = xdebug_hash_alloc(64, (xdebug_hash_dtor) xdebug_hash_brk_dtor);
-	context->class_breakpoints = xdebug_hash_alloc(64, (xdebug_hash_dtor) xdebug_hash_brk_dtor);
 	context->line_breakpoints = xdebug_llist_alloc((xdebug_llist_dtor) xdebug_llist_brk_dtor);
 	context->eval_id_lookup = xdebug_hash_alloc(64, (xdebug_hash_dtor) xdebug_hash_eval_info_dtor);
 	context->eval_id_sequence = 0;
@@ -2410,7 +2468,6 @@ int xdebug_dbgp_deinit(xdebug_con *context)
 	xdfree(context->options);
 	xdebug_hash_destroy(context->function_breakpoints);
 	xdebug_hash_destroy(context->exception_breakpoints);
-	xdebug_hash_destroy(context->class_breakpoints);
 	xdebug_hash_destroy(context->eval_id_lookup);
 	xdebug_llist_destroy(context->line_breakpoints, NULL);
 	xdebug_hash_destroy(context->breakpoint_list);
@@ -2470,8 +2527,12 @@ int xdebug_dbgp_error(xdebug_con *context, int type, char *exception_type, char
 	response = xdebug_xml_node_init("response");
 	xdebug_xml_add_attribute(response, "xmlns", "urn:debugger_protocol_v1");
 	xdebug_xml_add_attribute(response, "xmlns:xdebug", "http://xdebug.org/dbgp/xdebug");
-	xdebug_xml_add_attribute_ex(response, "command", XG(lastcmd), 0, 0);
-	xdebug_xml_add_attribute_ex(response, "transaction_id", XG(lasttransid), 0, 0);
+	/* lastcmd and lasttransid are not always set (for example when the
+	 * connection is severed before the first command is send) */
+	if (XG(lastcmd) && XG(lasttransid)) {
+		xdebug_xml_add_attribute_ex(response, "command", XG(lastcmd), 0, 0);
+		xdebug_xml_add_attribute_ex(response, "transaction_id", XG(lasttransid), 0, 0);
+	}
 	xdebug_xml_add_attribute(response, "status", xdebug_dbgp_status_strings[XG(status)]);
 	xdebug_xml_add_attribute(response, "reason", xdebug_dbgp_reason_strings[XG(reason)]);
 
@@ -2492,7 +2553,7 @@ int xdebug_dbgp_error(xdebug_con *context, int type, char *exception_type, char
 	return 1;
 }
 
-int xdebug_dbgp_breakpoint(xdebug_con *context, xdebug_llist *stack, const char *file, long lineno, int type, char *exception, char *message)
+int xdebug_dbgp_breakpoint(xdebug_con *context, xdebug_llist *stack, char *file, long lineno, int type, char *exception, char *message)
 {
 	xdebug_xml_node *response, *error_container;
 	TSRMLS_FETCH();
@@ -2503,8 +2564,12 @@ int xdebug_dbgp_breakpoint(xdebug_con *context, xdebug_llist *stack, const char
 	response = xdebug_xml_node_init("response");
 	xdebug_xml_add_attribute(response, "xmlns", "urn:debugger_protocol_v1");
 	xdebug_xml_add_attribute(response, "xmlns:xdebug", "http://xdebug.org/dbgp/xdebug");
-	xdebug_xml_add_attribute_ex(response, "command", XG(lastcmd), 0, 0);
-	xdebug_xml_add_attribute_ex(response, "transaction_id", XG(lasttransid), 0, 1);
+	/* lastcmd and lasttransid are not always set (for example when the
+	 * connection is severed before the first command is send) */
+	if (XG(lastcmd) && XG(lasttransid)) {
+		xdebug_xml_add_attribute_ex(response, "command", XG(lastcmd), 0, 0);
+		xdebug_xml_add_attribute_ex(response, "transaction_id", XG(lasttransid), 0, 0);
+	}
 	xdebug_xml_add_attribute(response, "status", xdebug_dbgp_status_strings[XG(status)]);
 	xdebug_xml_add_attribute(response, "reason", xdebug_dbgp_reason_strings[XG(reason)]);
 
@@ -2533,7 +2598,10 @@ int xdebug_dbgp_breakpoint(xdebug_con *context, xdebug_llist *stack, const char
 	xdebug_xml_node_dtor(response);
 
 	XG(lastcmd) = NULL;
-	XG(lasttransid) = NULL;
+	if (XG(lasttransid)) {
+		xdfree(XG(lasttransid));
+		XG(lasttransid) = NULL;
+	}
 
 	xdebug_dbgp_cmdloop(context, 1 TSRMLS_CC);
 
