@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007 Derick Rethans      |
+   | Copyright (c) 2002-2010 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -19,6 +19,15 @@
 #ifndef XDEBUG_PRIVATE_H
 #define XDEBUG_PRIVATE_H
 
+#include "zend.h"
+#include "zend_API.h"
+#include "zend_alloc.h"
+#include "zend_execute.h"
+#include "zend_compile.h"
+#include "zend_constants.h"
+#include "zend_extensions.h"
+#include "zend_exceptions.h"
+#include "zend_vm.h"
 #include "zend_hash.h"
 #include "xdebug_hash.h"
 #include "xdebug_llist.h"
@@ -166,6 +175,7 @@ typedef struct _function_stack_entry {
 	xdebug_llist *used_vars;
 	HashTable   *symbol_table;
 	zend_execute_data *execute_data;
+	zval        *This;
 
 	/* tracing properties */
 	signed long  memory;
