@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007 Derick Rethans      |
+   | Copyright (c) 2002-2010 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -58,11 +58,9 @@ char* xdebug_error_type(int type)
 		case E_USER_NOTICE:
 			return xdstrdup("Notice");
 			break;
-#ifdef ZEND_ENGINE_2
 		case E_STRICT:
 			return xdstrdup("Strict standards");
 			break;
-#endif
 #if PHP_VERSION_ID >= 50300
 		case E_DEPRECATED:
 			return xdstrdup("Deprecated");
@@ -74,6 +72,53 @@ char* xdebug_error_type(int type)
 	}
 }
 
+/*************************************************************************************************************************************/
+#define T(offset) (*(temp_variable *)((char *) Ts + offset))
+
+zval *xdebug_get_zval(zend_execute_data *zdata, znode *node, temp_variable *Ts, int *is_var)
+{
+	switch (node->op_type) {
+		case IS_CONST:
+			return &node->u.constant;
+			break;
+
+		case IS_TMP_VAR:
+			*is_var = 1;
+			return &T(node->u.var).tmp_var;
+			break;
+
+		case IS_VAR:
+			*is_var = 1;
+			if (T(node->u.var).var.ptr) {
+				return T(node->u.var).var.ptr;
+			} else {
+				fprintf(stderr, "\nIS_VAR\n");
+			}
+			break;
+
+		case IS_CV: {
+			zval **tmp;
+			tmp = zend_get_compiled_variable_value(zdata, node->u.constant.value.lval);
+			if (tmp) {
+				return *tmp;
+			}
+			break;
+		}
+
+		case IS_UNUSED:
+			fprintf(stderr, "\nIS_UNUSED\n");
+			break;
+
+		default:
+			fprintf(stderr, "\ndefault %d\n", node->op_type);
+			break;
+	}
+
+	*is_var = 1;
+
+	return NULL;
+}
+
 /*****************************************************************************
 ** PHP Variable related utility functions
 */
@@ -102,19 +147,19 @@ zval* xdebug_get_php_symbol(char* name, int name_length)
 	return NULL;
 }
 
-static char* xdebug_get_property_info(char *mangled_property, int mangled_len, char **property_name)
+static char* xdebug_get_property_info(char *mangled_property, int mangled_len, char **property_name, char **class_name)
 {
-#ifdef ZEND_ENGINE_2
-	char *prop_name, *class_name;
+	char *prop_name, *cls_name;
 
 #if PHP_VERSION_ID >= 50200
-	zend_unmangle_property_name(mangled_property, mangled_len - 1, &class_name, &prop_name);
+	zend_unmangle_property_name(mangled_property, mangled_len - 1, &cls_name, &prop_name);
 #else
-	zend_unmangle_property_name(mangled_property, &class_name, &prop_name);
+	zend_unmangle_property_name(mangled_property, &cls_name, &prop_name);
 #endif
 	*property_name = prop_name;
-	if (class_name) {
-		if (class_name[0] == '*') {
+	*class_name = cls_name;
+	if (cls_name) {
+		if (cls_name[0] == '*') {
 			return "protected";
 		} else {
 			return "private";
@@ -122,14 +167,10 @@ static char* xdebug_get_property_info(char *mangled_property, int mangled_len, c
 	} else {
 		return "public";
 	}
-#else
-	*property_name = mangled_property;
-	return "var";
-#endif
 }
 
 
-static xdebug_var_export_options* get_options_from_ini(TSRMLS_D)
+xdebug_var_export_options* xdebug_var_export_options_from_ini(TSRMLS_D)
 {
 	xdebug_var_export_options *options;
 	options = xdmalloc(sizeof(xdebug_var_export_options));
@@ -158,11 +199,12 @@ static xdebug_var_export_options* get_options_from_ini(TSRMLS_D)
 	}
 
 	options->runtime = (xdebug_var_runtime_page*) xdmalloc((options->max_depth + 1) * sizeof(xdebug_var_runtime_page));
+	options->no_decoration = 0;
 
 	return options;
 }
 
-xdebug_var_export_options xdebug_var_nolimit_options = { 1048576, 1048576, 64, 1, NULL };
+xdebug_var_export_options xdebug_var_nolimit_options = { 1048576, 1048576, 64, 1, NULL, 0 };
 
 xdebug_var_export_options* xdebug_var_get_nolimit_options(TSRMLS_D)
 {
@@ -223,7 +265,7 @@ static int xdebug_object_element_export(zval **zv XDEBUG_ZEND_HASH_APPLY_TSRMLS_
 	int level, debug_zval;
 	xdebug_str *str;
 	xdebug_var_export_options *options;
-	char *prop_name, *modifier;
+	char *prop_name, *class_name, *modifier, *prop_class_name;
 #if !defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300
 	TSRMLS_FETCH();
 #endif
@@ -232,13 +274,18 @@ static int xdebug_object_element_export(zval **zv XDEBUG_ZEND_HASH_APPLY_TSRMLS_
 	str        = va_arg(args, struct xdebug_str*);
 	debug_zval = va_arg(args, int);
 	options    = va_arg(args, xdebug_var_export_options*);
+	class_name = va_arg(args, char *);
 
 	if (options->runtime[level].current_element_nr >= options->runtime[level].start_element_nr &&
 		options->runtime[level].current_element_nr < options->runtime[level].end_element_nr)
 	{
 		if (hash_key->nKeyLength != 0) {
-			modifier = xdebug_get_property_info(hash_key->arKey, hash_key->nKeyLength, &prop_name);
-			xdebug_str_add(str, xdebug_sprintf("%s $%s = ", modifier, prop_name), 1);
+			modifier = xdebug_get_property_info(hash_key->arKey, hash_key->nKeyLength, &prop_name, &prop_class_name);
+			if (strcmp(modifier, "private") != 0 || strcmp(class_name, prop_class_name) == 0) {
+				xdebug_str_add(str, xdebug_sprintf("%s $%s = ", modifier, prop_name), 1);
+			} else {
+				xdebug_str_add(str, xdebug_sprintf("%s ${%s}:%s = ", modifier, prop_class_name, prop_name), 1);
+			}
 		}
 		xdebug_var_export(zv, str, level + 2, debug_zval, options TSRMLS_CC);
 		xdebug_str_addl(str, "; ", 2, 0);
@@ -281,7 +328,9 @@ void xdebug_var_export(zval **struc, xdebug_str *str, int level, int debug_zval,
 
 		case IS_STRING:
 			tmp_str = php_addcslashes(Z_STRVAL_PP(struc), Z_STRLEN_PP(struc), &tmp_len, 0, "'\\\0..\37", 6 TSRMLS_CC);
-			if (options->max_data == 0 || Z_STRLEN_PP(struc) <= options->max_data) {
+			if (options->no_decoration) {
+				xdebug_str_add(str, tmp_str, 0);
+			} else if (options->max_data == 0 || Z_STRLEN_PP(struc) <= options->max_data) {
 				xdebug_str_add(str, xdebug_sprintf("'%s'", tmp_str), 1);
 			} else {
 				xdebug_str_addl(str, "'", 1, 0);
@@ -317,13 +366,18 @@ void xdebug_var_export(zval **struc, xdebug_str *str, int level, int debug_zval,
 		case IS_OBJECT:
 			myht = Z_OBJPROP_PP(struc);
 			if (myht->nApplyCount < 1) {
-				xdebug_str_add(str, xdebug_sprintf("class %s { ", Z_OBJCE_PP(struc)->name), 1);
+				char *class_name;
+				zend_uint class_name_len;
+
+				zend_get_object_classname(*struc, &class_name, &class_name_len TSRMLS_CC);
+				xdebug_str_add(str, xdebug_sprintf("class %s { ", class_name), 1);
+
 				if (level <= options->max_depth) {
 					options->runtime[level].current_element_nr = 0;
 					options->runtime[level].start_element_nr = 0;
 					options->runtime[level].end_element_nr = options->max_children;
 
-					zend_hash_apply_with_arguments(myht XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_object_element_export, 4, level, str, debug_zval, options);
+					zend_hash_apply_with_arguments(myht XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_object_element_export, 5, level, str, debug_zval, options, class_name);
 					/* Remove the ", " at the end of the string */
 					if (myht->nNumOfElements > 0) {
 						xdebug_str_chop(str, 2);
@@ -332,6 +386,7 @@ void xdebug_var_export(zval **struc, xdebug_str *str, int level, int debug_zval,
 					xdebug_str_addl(str, "...", 3, 0);
 				}
 				xdebug_str_addl(str, " }", 2, 0);
+				efree(class_name);
 			} else {
 				xdebug_str_addl(str, "...", 3, 0);
 			}
@@ -358,7 +413,7 @@ char* xdebug_get_zval_value(zval *val, int debug_zval, xdebug_var_export_options
 	TSRMLS_FETCH();
 
 	if (!options) {
-		options = get_options_from_ini(TSRMLS_C);
+		options = xdebug_var_export_options_from_ini(TSRMLS_C);
 		default_options = 1;
 	}
 
@@ -408,9 +463,14 @@ static void xdebug_var_synopsis(zval **struc, xdebug_str *str, int level, int de
 			xdebug_str_add(str, xdebug_sprintf("array(%d)", myht->nNumOfElements), 1);
 			break;
 
-		case IS_OBJECT:
-			xdebug_str_add(str, xdebug_sprintf("class %s", Z_OBJCE_PP(struc)->name), 1);
+		case IS_OBJECT: {
+			char *class_name;
+			zend_uint class_name_len;
+
+			zend_get_object_classname(*struc, &class_name, &class_name_len TSRMLS_CC);
+			xdebug_str_add(str, xdebug_sprintf("class %s", class_name), 1);
 			break;
+		}
 
 		case IS_RESOURCE: {
 			char *type_name;
@@ -429,161 +489,20 @@ char* xdebug_get_zval_synopsis(zval *val, int debug_zval, xdebug_var_export_opti
 	TSRMLS_FETCH();
 
 	if (!options) {
-		options = get_options_from_ini(TSRMLS_C);
+		options = xdebug_var_export_options_from_ini(TSRMLS_C);
 		default_options = 1;
 	}
 
 	xdebug_var_synopsis(&val, (xdebug_str*) &str, 1, debug_zval, options TSRMLS_CC);
 
 	if (default_options) {
+		xdfree(options->runtime);
 		xdfree(options);
 	}
 
 	return str.d;
 }
 
-/*****************************************************************************
-** XML variable printing routines
-*/
-
-static int xdebug_array_element_export_xml(zval **zv XDEBUG_ZEND_HASH_APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
-{
-	int level;
-	xdebug_str *str;
-#if !defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300
-	TSRMLS_FETCH();
-#endif
-
-	level = va_arg(args, int);
-	str   = va_arg(args, struct xdebug_str*);
-
-	xdebug_str_addl(str, "<var", 4, 0);
-	if (hash_key->nKeyLength == 0) { /* numeric key */
-		xdebug_str_add(str, xdebug_sprintf(" name='%ld'", hash_key->h), 1);
-	} else { /* string key */
-		xdebug_str_add(str, xdebug_sprintf(" name='%s'", hash_key->arKey), 1);
-	}
-	xdebug_str_add(str, xdebug_sprintf(" id='%p'>", *zv), 1);
-	xdebug_var_export_xml(zv, str, level + 1 TSRMLS_CC);
-	xdebug_str_addl(str, "</var>", 6, 0);
-	return 0;
-}
-
-static int xdebug_object_element_export_xml(zval **zv XDEBUG_ZEND_HASH_APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
-{
-	int level;
-	xdebug_str *str;
-	char *prop_name, *modifier;
-#if !defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300
-	TSRMLS_FETCH();
-#endif
-
-	level = va_arg(args, int);
-	str   = va_arg(args, struct xdebug_str*);
-
-	xdebug_str_addl(str, "<var", 4, 0);
-	if (hash_key->nKeyLength != 0) {
-		modifier = xdebug_get_property_info(hash_key->arKey, hash_key->nKeyLength, &prop_name);
-		xdebug_str_add(str, xdebug_sprintf(" name='%s' facet='%s'", prop_name, modifier), 1);
-	}
-	xdebug_str_add(str, xdebug_sprintf(" id='%p'>", *zv), 1);
-	xdebug_var_export_xml(zv, str, level + 1 TSRMLS_CC);
-	xdebug_str_addl(str, "</var>", 6, 0);
-	return 0;
-}
-
-void xdebug_var_export_xml(zval **struc, xdebug_str *str, int level TSRMLS_DC)
-{
-	HashTable *myht;
-	char*     tmp_str;
-	int       newlen;
-
-	if (!*struc) {
-		xdebug_str_addl(str, "<uninitialized/>", 16, 0);
-		return;
-	}
-	
-	switch (Z_TYPE_PP(struc)) {
-		case IS_BOOL:
-			xdebug_str_add(str, xdebug_sprintf("<bool>%s</bool>", Z_LVAL_PP(struc) ? "1" : "0"), 1);
-			break;
-
-		case IS_NULL:
-			xdebug_str_addl(str, "<null/>", 7, 0);
-			break;
-
-		case IS_LONG:
-			xdebug_str_add(str, xdebug_sprintf("<int>%ld</int>", Z_LVAL_PP(struc)), 1);
-			break;
-
-		case IS_DOUBLE:
-			xdebug_str_add(str, xdebug_sprintf("<float>%.*G</float>", (int) EG(precision), Z_DVAL_PP(struc)), 1);
-			break;
-
-		case IS_STRING:
-			xdebug_str_addl(str, "<string>", 8, 0);
-			tmp_str = xdebug_xmlize(Z_STRVAL_PP(struc), Z_STRLEN_PP(struc), &newlen);
-			xdebug_str_addl(str, tmp_str, newlen, 0);
-			efree(tmp_str);
-			xdebug_str_addl(str, "</string>", 9, 0);
-			break;
-
-		case IS_ARRAY:
-			myht = Z_ARRVAL_PP(struc);
-			if (myht->nApplyCount < 1) {
-				xdebug_str_addl(str, "<array>", 7, 0);
-				zend_hash_apply_with_arguments(myht XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_array_element_export_xml, 2, level, str);
-				xdebug_str_addl(str, "</array>", 8, 0);
-			} else {
-				xdebug_str_addl(str, "<array hidden='true' recursive='true'/>", 39, 0);
-			}
-			break;
-
-		case IS_OBJECT:
-			myht = Z_OBJPROP_PP(struc);
-			if (myht->nApplyCount < 1) {
-				xdebug_str_add(str, xdebug_sprintf("<object class='%s'>", Z_OBJCE_PP(struc)->name), 1);
-				zend_hash_apply_with_arguments(myht XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_object_element_export_xml, 2, level, str);
-				xdebug_str_addl(str, "</object>", 9, 0);
-			} else {
-				xdebug_str_addl(str, "<object hidden='true' recursive='true'/>", 40, 0);
-			}
-			break;
-
-		case IS_RESOURCE: {
-			char *type_name;
-
-			type_name = zend_rsrc_list_get_rsrc_type(Z_LVAL_PP(struc) TSRMLS_CC);
-			xdebug_str_add(str, xdebug_sprintf("<resource id='%ld' type='%s'/>", Z_LVAL_PP(struc), type_name ? type_name : "Unknown"), 1);
-			break;
-		}
-
-		default:
-			xdebug_str_addl(str, "<null/>", 7, 0);
-			break;
-	}
-}
-
-char* xdebug_get_zval_value_xml(char *name, zval *val)
-{
-	xdebug_str str = {0, 0, NULL};
-	TSRMLS_FETCH();
-
-	if (name) {
-		xdebug_str_addl(&str, "<var name='", 11, 0);
-		xdebug_str_add(&str, name, 0);
-		xdebug_str_add(&str, xdebug_sprintf("' id='%p'>", val), 1);
-	} else {
-		xdebug_str_add(&str, xdebug_sprintf("<var id='%p'>", val), 1);
-	}
-	
-	xdebug_var_export_xml(&val, (xdebug_str*) &str, 1 TSRMLS_CC);
-
-	xdebug_str_addl(&str, "</var>", 7, 0);
-
-	return str.d;
-}
-
 /*****************************************************************************
 ** XML node printing routines
 */
@@ -594,7 +513,9 @@ static int xdebug_array_element_export_xml_node(zval **zv XDEBUG_ZEND_HASH_APPLY
 	xdebug_xml_node *parent;
 	xdebug_xml_node *node;
 	xdebug_var_export_options *options;
-	char *name = NULL, *parent_name = NULL, *full_name = NULL;
+	char *name = NULL, *parent_name = NULL;
+	int   name_len = 0;
+	xdebug_str full_name = { 0, 0, NULL };
 #if !defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300
 	TSRMLS_FETCH();
 #endif
@@ -610,33 +531,37 @@ static int xdebug_array_element_export_xml_node(zval **zv XDEBUG_ZEND_HASH_APPLY
 		node = xdebug_xml_node_init("property");
 	
 		if (hash_key->nKeyLength != 0) {
-			name = xdstrdup(hash_key->arKey);
-			if (!parent_name) {
-				full_name = NULL;
-			} else if (parent_name[0] != '$') {
-				full_name = xdebug_sprintf("$%s['%s']", parent_name, name);
-			} else {
-				full_name = xdebug_sprintf("%s['%s']", parent_name, name);
+			name = xdstrndup(hash_key->arKey, hash_key->nKeyLength);
+			name_len = hash_key->nKeyLength - 1;
+			if (parent_name) {
+				if (parent_name[0] != '$') {
+					xdebug_str_addl(&full_name, "$", 1, 0);
+				}
+				xdebug_str_add(&full_name, parent_name, 0);
+				xdebug_str_addl(&full_name, "['", 2, 0);
+				xdebug_str_addl(&full_name, name, name_len, 0);
+				xdebug_str_addl(&full_name, "']", 2, 0);
 			}
 		} else {
 			name = xdebug_sprintf("%ld", hash_key->h);
-			if (!parent_name) {
-				full_name = NULL;
-			} else if (parent_name[0] != '$') {
-				full_name = xdebug_sprintf("$%s[%s]", parent_name, name);
-			} else {
-				full_name = xdebug_sprintf("%s[%s]", parent_name, name);
+			name_len = strlen(name);
+			if (parent_name) {
+				if (parent_name[0] != '$') {
+					xdebug_str_add(&full_name, xdebug_sprintf("$%s[%s]", parent_name, name), 1);
+				} else {
+					xdebug_str_add(&full_name, xdebug_sprintf("%s[%s]", parent_name, name), 1);
+				}
 			}
 		}
 
-		xdebug_xml_add_attribute_ex(node, "name", name, 0, 1);
-		if (full_name) {
-			xdebug_xml_add_attribute_ex(node, "fullname", full_name, 0, 1);
+		xdebug_xml_add_attribute_exl(node, "name", 4, name, name_len, 0, 1);
+		if (full_name.l) {
+			xdebug_xml_add_attribute_exl(node, "fullname", 8, full_name.d, full_name.l, 0, 1);
 		}
 		xdebug_xml_add_attribute_ex(node, "address", xdebug_sprintf("%ld", (long) *zv), 0, 1);
 
 		xdebug_xml_add_child(parent, node);
-		xdebug_var_export_xml_node(zv, full_name, node, options, level + 1 TSRMLS_CC);
+		xdebug_var_export_xml_node(zv, full_name.d, node, options, level + 1 TSRMLS_CC);
 	}
 	options->runtime[level].current_element_nr++;
 	return 0;
@@ -648,7 +573,7 @@ static int xdebug_object_element_export_xml_node(zval **zv XDEBUG_ZEND_HASH_APPL
 	xdebug_xml_node *parent;
 	xdebug_xml_node *node;
 	xdebug_var_export_options *options;
-	char *prop_name, *modifier;
+	char *prop_name, *modifier, *class_name, *prop_class_name;
 	char *parent_name = NULL, *full_name = NULL;
 #if !defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300
 	TSRMLS_FETCH();
@@ -658,31 +583,38 @@ static int xdebug_object_element_export_xml_node(zval **zv XDEBUG_ZEND_HASH_APPL
 	parent = va_arg(args, xdebug_xml_node*);
 	full_name = parent_name = va_arg(args, char *);
 	options = va_arg(args, xdebug_var_export_options*);
+	class_name = va_arg(args, char *);
 
 	if (options->runtime[level].current_element_nr >= options->runtime[level].start_element_nr &&
 		options->runtime[level].current_element_nr < options->runtime[level].end_element_nr)
 	{
-		node = xdebug_xml_node_init("property");
-		
 		if (hash_key->nKeyLength != 0) {
-			modifier = xdebug_get_property_info(hash_key->arKey, hash_key->nKeyLength, &prop_name);
-			xdebug_xml_add_attribute(node, "name", prop_name);
-			/* XXX static vars? */
+			modifier = xdebug_get_property_info(hash_key->arKey, hash_key->nKeyLength, &prop_name, &prop_class_name);
+			if (strcmp(modifier, "private") != 0 || strcmp(class_name, prop_class_name) == 0) {
+				node = xdebug_xml_node_init("property");
+				xdebug_xml_add_attribute_ex(node, "name", xdstrdup(prop_name), 0, 1);
+			} else {
+				return 0;
+			}
 
-			if (parent_name) {
-				if (parent_name[0] != '$') {
-					full_name = xdebug_sprintf("$%s->%s", parent_name, prop_name);
-				} else {
-					full_name = xdebug_sprintf("%s->%s", parent_name, prop_name);
+			if (strcmp(modifier, "private") != 0 || strcmp(class_name, prop_class_name) == 0) {
+				if (parent_name) {
+					if (parent_name[0] != '$') {
+						full_name = xdebug_sprintf("$%s->%s", parent_name, prop_name);
+					} else {
+						full_name = xdebug_sprintf("%s->%s", parent_name, prop_name);
+					}
+					xdebug_xml_add_attribute_ex(node, "fullname", full_name, 0, 1);
 				}
-				xdebug_xml_add_attribute_ex(node, "fullname", full_name, 0, 1);
 			}
 			xdebug_xml_add_attribute(node, "facet", modifier);
-		}
-		xdebug_xml_add_attribute_ex(node, "address", xdebug_sprintf("%ld", (long) *zv), 0, 1);
 
-		xdebug_xml_add_child(parent, node);
-		xdebug_var_export_xml_node(zv, full_name, node, options, level + 1 TSRMLS_CC);
+			xdebug_xml_add_attribute_ex(node, "address", xdebug_sprintf("%ld", (long) *zv), 0, 1);
+
+			xdebug_xml_add_child(parent, node);
+
+			xdebug_var_export_xml_node(zv, full_name, node, options, level + 1 TSRMLS_CC);
+		}
 	}
 	options->runtime[level].current_element_nr++;
 	return 0;
@@ -691,10 +623,8 @@ static int xdebug_object_element_export_xml_node(zval **zv XDEBUG_ZEND_HASH_APPL
 void xdebug_var_export_xml_node(zval **struc, char *name, xdebug_xml_node *node, xdebug_var_export_options *options, int level TSRMLS_DC)
 {
 	HashTable *myht;
-#ifdef ZEND_ENGINE_2
 	char *class_name;
 	zend_uint class_name_len;
-#endif
 
 	switch (Z_TYPE_PP(struc)) {
 		case IS_BOOL:
@@ -723,7 +653,7 @@ void xdebug_var_export_xml_node(zval **struc, char *name, xdebug_xml_node *node,
 			} else {
 				xdebug_xml_add_text_encodel(node, xdstrndup(Z_STRVAL_PP(struc), options->max_data), options->max_data);
 			}
-			xdebug_xml_add_attribute(node, "size", xdebug_sprintf("%d", Z_STRLEN_PP(struc)));
+			xdebug_xml_add_attribute_ex(node, "size", xdebug_sprintf("%d", Z_STRLEN_PP(struc)), 0, 1);
 			break;
 
 		case IS_ARRAY:
@@ -733,10 +663,10 @@ void xdebug_var_export_xml_node(zval **struc, char *name, xdebug_xml_node *node,
 			if (myht->nApplyCount < 1) {
 				xdebug_xml_add_attribute_ex(node, "numchildren", xdebug_sprintf("%d", myht->nNumOfElements), 0, 1);
 				if (level < options->max_depth) {
+					xdebug_xml_add_attribute_ex(node, "page", xdebug_sprintf("%d", options->runtime[level].page), 0, 1);
+					xdebug_xml_add_attribute_ex(node, "pagesize", xdebug_sprintf("%d", options->max_children), 0, 1);
 					options->runtime[level].current_element_nr = 0;
-					if (level == 0 && myht->nNumOfElements > options->max_children) {
-						xdebug_xml_add_attribute_ex(node, "page", xdebug_sprintf("%d", options->runtime[level].page), 0, 1);
-						xdebug_xml_add_attribute_ex(node, "pagesize", xdebug_sprintf("%d", options->max_children), 0, 1);
+					if (level == 0) {
 						options->runtime[level].start_element_nr = options->max_children * options->runtime[level].page;
 						options->runtime[level].end_element_nr = options->max_children * (options->runtime[level].page + 1);
 					} else {
@@ -752,35 +682,44 @@ void xdebug_var_export_xml_node(zval **struc, char *name, xdebug_xml_node *node,
 
 		case IS_OBJECT:
 			myht = Z_OBJPROP_PP(struc);
+
 			xdebug_xml_add_attribute(node, "type", "object");
-			xdebug_xml_add_attribute(node, "children", (myht && zend_hash_num_elements(myht))?"1":"0");
-#ifdef ZEND_ENGINE_2
-			Z_OBJ_HANDLER(**struc, get_class_name)(*struc, &class_name, &class_name_len, 0 TSRMLS_CC);
+			zend_get_object_classname(*struc, &class_name, &class_name_len TSRMLS_CC);
 			xdebug_xml_add_attribute_ex(node, "classname", xdstrdup(class_name), 0, 1);
-			efree(class_name);
-#else
-			xdebug_xml_add_attribute_ex(node, "classname", xdstrdup(Z_OBJCE_PP(struc)->name), 0, 1);
-#endif
+			xdebug_xml_add_attribute(node, "children", (myht && zend_hash_num_elements(myht))?"1":"0");
+
+			/** Temporary additional property **/
+			{
+				xdebug_xml_node *pnode;
+
+				pnode = xdebug_xml_node_init("property");
+				xdebug_xml_add_attribute(pnode, "name", "CLASSNAME");
+				xdebug_xml_add_attribute(pnode, "type", "string");
+				xdebug_xml_add_text(pnode, xdstrdup(class_name));
+				xdebug_xml_add_child(node, pnode);
+			}
+
 			if (myht) {
 				if (myht->nApplyCount < 1) {
 					xdebug_xml_add_attribute_ex(node, "numchildren", xdebug_sprintf("%d", zend_hash_num_elements(myht)), 0, 1);
 					if (level < options->max_depth) {
+						xdebug_xml_add_attribute_ex(node, "page", xdebug_sprintf("%d", options->runtime[level].page), 0, 1);
+						xdebug_xml_add_attribute_ex(node, "pagesize", xdebug_sprintf("%d", options->max_children), 0, 1);
 						options->runtime[level].current_element_nr = 0;
-						if (level == 0 && myht->nNumOfElements > options->max_children) {
-							xdebug_xml_add_attribute_ex(node, "page", xdebug_sprintf("%d", options->runtime[level].page), 0, 1);
-							xdebug_xml_add_attribute_ex(node, "pagesize", xdebug_sprintf("%d", options->max_children), 0, 1);
+						if (level == 0) {
 							options->runtime[level].start_element_nr = options->max_children * options->runtime[level].page;
 							options->runtime[level].end_element_nr = options->max_children * (options->runtime[level].page + 1);
 						} else {
 							options->runtime[level].start_element_nr = 0;
 							options->runtime[level].end_element_nr = options->max_children;
 						}
-						zend_hash_apply_with_arguments(myht XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_object_element_export_xml_node, 4, level, node, name, options);
+						zend_hash_apply_with_arguments(myht XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_object_element_export_xml_node, 5, level, node, name, options, class_name);
 					}
 				} else {
 					xdebug_xml_add_attribute(node, "recursive", "1");
 				}
 			}
+			efree(class_name);
 			break;
 
 		case IS_RESOURCE: {
@@ -812,7 +751,7 @@ xdebug_xml_node* xdebug_get_zval_value_xml_node(char *name, zval *val, xdebug_va
 			full_name = xdstrdup(name);
 		}
 		xdebug_xml_add_attribute_ex(node, "name", xdstrdup(name), 0, 1);
-		xdebug_xml_add_attribute_ex(node, "fullname", xdstrdup(full_name), 0, 1);
+		xdebug_xml_add_attribute_ex(node, "fullname", full_name, 0, 1);
 	}
 	xdebug_xml_add_attribute_ex(node, "address", xdebug_sprintf("%ld", (long) val), 0, 1);
 	xdebug_var_export_xml_node(&val, name, node, options, 0 TSRMLS_CC);
@@ -880,7 +819,7 @@ static int xdebug_object_element_export_fancy(zval **zv XDEBUG_ZEND_HASH_APPLY_T
 	xdebug_str *str;
 	xdebug_var_export_options *options;
 	char *key;
-	char *prop_name, *modifier;
+	char *prop_name, *class_name, *modifier, *prop_class_name;
 #if !defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300
 	TSRMLS_FETCH();
 #endif
@@ -889,6 +828,7 @@ static int xdebug_object_element_export_fancy(zval **zv XDEBUG_ZEND_HASH_APPLY_T
 	str        = va_arg(args, struct xdebug_str*);
 	debug_zval = va_arg(args, int);
 	options    = va_arg(args, xdebug_var_export_options*);
+	class_name = va_arg(args, char *);
 
 	if (options->runtime[level].current_element_nr >= options->runtime[level].start_element_nr &&
 		options->runtime[level].current_element_nr < options->runtime[level].end_element_nr)
@@ -897,8 +837,12 @@ static int xdebug_object_element_export_fancy(zval **zv XDEBUG_ZEND_HASH_APPLY_T
 
 		key = hash_key->arKey;
 		if (hash_key->nKeyLength != 0) {
-			modifier = xdebug_get_property_info(hash_key->arKey, hash_key->nKeyLength, &prop_name);
-			xdebug_str_add(str, xdebug_sprintf("<i>%s</i> '%s' <font color='%s'>=&gt;</font> ", modifier, prop_name, COLOR_POINTER), 1);
+			modifier = xdebug_get_property_info(hash_key->arKey, hash_key->nKeyLength, &prop_name, &prop_class_name);
+			if (strcmp(modifier, "private") != 0 || strcmp(class_name, prop_class_name) == 0) {
+				xdebug_str_add(str, xdebug_sprintf("<i>%s</i> '%s' <font color='%s'>=&gt;</font> ", modifier, prop_name, COLOR_POINTER), 1);
+			} else {
+				xdebug_str_add(str, xdebug_sprintf("<i>%s</i> '%s' <small>(%s)</small> <font color='%s'>=&gt;</font> ", modifier, prop_name, prop_class_name, COLOR_POINTER), 1);
+			}
 		}
 		xdebug_var_export_fancy(zv, str, level + 1, debug_zval, options TSRMLS_CC);
 	}
@@ -973,8 +917,7 @@ void xdebug_var_export_fancy(zval **struc, xdebug_str *str, int level, int debug
 						xdebug_str_add(str, xdebug_sprintf("<i><font color='%s'>empty</font></i>\n", COLOR_EMPTY), 1);
 					}
 				} else {
-					xdebug_str_add(str, xdebug_sprintf("%*s", (level * 4) - 2, ""), 1);
-					xdebug_str_add(str, xdebug_sprintf("...\n"), 1);
+					xdebug_str_add(str, xdebug_sprintf("%*s...\n", (level * 4) - 2, ""), 1);
 				}
 			} else {
 				xdebug_str_addl(str, "<i>&</i><b>array</b>\n", 21, 0);
@@ -986,28 +929,19 @@ void xdebug_var_export_fancy(zval **struc, xdebug_str *str, int level, int debug
 			xdebug_str_add(str, xdebug_sprintf("\n%*s", (level - 1) * 4, ""), 1);
 			if (myht->nApplyCount < 1) {
 				xdebug_str_add(str, xdebug_sprintf("<b>object</b>(<i>%s</i>)", Z_OBJCE_PP(struc)->name), 1);
-#ifdef ZEND_ENGINE_2
 				xdebug_str_add(str, xdebug_sprintf("[<i>%d</i>]\n", Z_OBJ_HANDLE_PP(struc)), 1);
-#else
-				xdebug_str_addl(str, "\n", 1, 0);
-#endif
 				if (level <= options->max_depth) {
 					options->runtime[level].current_element_nr = 0;
 					options->runtime[level].start_element_nr = 0;
 					options->runtime[level].end_element_nr = options->max_children;
 
-					zend_hash_apply_with_arguments(myht XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_object_element_export_fancy, 4, level, str, debug_zval, options);
+					zend_hash_apply_with_arguments(myht XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_object_element_export_fancy, 5, level, str, debug_zval, options, Z_OBJCE_PP(struc)->name);
 				} else {
-					xdebug_str_add(str, xdebug_sprintf("%*s", (level * 4) - 2, ""), 1);
-					xdebug_str_add(str, xdebug_sprintf("...\n"), 1);
+					xdebug_str_add(str, xdebug_sprintf("%*s...\n", (level * 4) - 2, ""), 1);
 				}
 			} else {
 				xdebug_str_add(str, xdebug_sprintf("<i>&</i><b>object</b>(<i>%s</i>)", Z_OBJCE_PP(struc)->name), 1);
-#ifdef ZEND_ENGINE_2
 				xdebug_str_add(str, xdebug_sprintf("[<i>%d</i>]\n", Z_OBJ_HANDLE_PP(struc)), 1);
-#else
-				xdebug_str_addl(str, "\n", 1, 0);
-#endif
 			}
 			break;
 
@@ -1034,7 +968,7 @@ char* xdebug_get_zval_value_fancy(char *name, zval *val, int *len, int debug_zva
 	int default_options = 0;
 
 	if (!options) {
-		options = get_options_from_ini(TSRMLS_C);
+		options = xdebug_var_export_options_from_ini(TSRMLS_C);
 		default_options = 1;
 	}
 
@@ -1043,6 +977,7 @@ char* xdebug_get_zval_value_fancy(char *name, zval *val, int *len, int debug_zva
 	xdebug_str_addl(&str, "</pre>", 6, 0);
 
 	if (default_options) {
+		xdfree(options->runtime);
 		xdfree(options);
 	}
 
@@ -1085,9 +1020,7 @@ static void xdebug_var_synopsis_fancy(zval **struc, xdebug_str *str, int level,
 
 		case IS_OBJECT:
 			xdebug_str_add(str, xdebug_sprintf("<font color='%s'>object(%s)", COLOR_OBJECT, Z_OBJCE_PP(struc)->name), 1);
-#ifdef ZEND_ENGINE_2
 			xdebug_str_add(str, xdebug_sprintf("[%d]", Z_OBJ_HANDLE_PP(struc)), 1);
-#endif
 			xdebug_str_addl(str, "</font>", 7, 0);
 			break;
 
@@ -1107,13 +1040,14 @@ char* xdebug_get_zval_synopsis_fancy(char *name, zval *val, int *len, int debug_
 	int default_options = 0;
 
 	if (!options) {
-		options = get_options_from_ini(TSRMLS_C);
+		options = xdebug_var_export_options_from_ini(TSRMLS_C);
 		default_options = 1;
 	}
 
 	xdebug_var_synopsis_fancy(&val, (xdebug_str*) &str, 1, debug_zval, options TSRMLS_CC);
 
 	if (default_options) {
+		xdfree(options->runtime);
 		xdfree(options);
 	}
 
